// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#ifndef __ESWIN_ACCELERATOR_KIT_TYPES_H__
#define __ESWIN_ACCELERATOR_KIT_TYPES_H__

#include <stdint.h>
#include "es_type.h"
#include "es_nn_common.h"
#include <linux/es_vb_user.h>

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif

typedef struct AK_Capability_S {
    ES_U64 reserved;
} ES_AK_Capability_S;

typedef enum AK_LOG_LEVEL_E {
    ES_AK_LOG_ERROR = 0x1,
    ES_AK_LOG_WARNING = 0x2,
    ES_AK_LOG_INFO = 0x3,
    ES_AK_LOG_DEBUG = 0x4,
    ES_AK_LOG_NONE
} ES_AK_LOG_LEVEL_E;

typedef enum AK_DEVICE_E {
    ES_AK_DEV_DSP_0 = 0x0,
    ES_AK_DEV_DSP_1 = 0x1,
    ES_AK_DEV_DSP_2 = 0x2,
    ES_AK_DEV_DSP_3 = 0x3,
    ES_AK_DEV_DSP_4 = 0x4,
    ES_AK_DEV_DSP_5 = 0x5,
    ES_AK_DEV_DSP_6 = 0x6,
    ES_AK_DEV_DSP_7 = 0x7,
    ES_AK_DEV_HAE_0 = 0x8,
    ES_AK_DEV_HAE_1 = 0x9,
    ES_AK_DEV_GPU_0 = 0xa,
    ES_AK_DEV_GPU_1 = 0xb,
    ES_AK_DEV_DIE0_CPU = 0xc,
    ES_AK_DEV_DIE1_CPU = 0xd,
    ES_AK_DEV_BUTT
} ES_AK_DEVICE_E;

typedef enum {
    BORDER_CONSTANT = 0,
    BORDER_REPLICATE = 1,
    BORDER_REFLECT = 2,
    BORDER_WRAP = 3,
    BORDER_REFLECT_101 = 4,
    BORDER_TRANSPARENT = 5,
    BORDER_ISOLATED = 16
} ES_BORDER_TYPES_E;

/**
 * @brief Structure representing a single point in pose estimation.
 */
typedef struct {
    int32_t x;      /**< X-coordinate of the point in pixels. */
    int32_t y;      /**< Y-coordinate of the point in pixels. */
    float score;    /**< Confidence score for the detected point. */
} ES_POSE_POINT;

/**
 * @brief Configuration structure for detection output parameters.
 */
typedef struct {
    ES_S32 anchorsNum;                     /**< Number of anchor boxes used in the detection network. */
    ES_FLOAT anchorScale[MAX_TOTAL_ANCHORS_NUM * 2]; /**< Array of anchor scales, defined as pairs of width and height ratios. */
    ES_S32 imgH;                           /**< Height of the input image in pixels. */
    ES_S32 imgW;                           /**< Width of the input image in pixels. */
    ES_S32 clsNum;                         /**< Number of object classes for detection. */
    ES_FLOAT inputScale[MAX_IN_TENSOR_NUM];/**< Array of input tensor scales for normalization. */
    ES_NMS_METHOD_E nmsMethod;             /**< Non-maximum suppression (NMS) method to filter overlapping detections. */
    ES_IOU_METHOD_E iouMethod;             /**< Intersection over union (IOU) method for NMS calculations. */
    ES_BOX_TYPE_E outBoxType;              /**< Type of output bounding box format (e.g., corner or center coordinates). */
    ES_BOOL coordNorm;                     /**< Flag indicating whether box coordinates are normalized to [0, 1]. */
    ES_S32 maxBoxesPerClass;               /**< Maximum number of boxes to retain per class after NMS. */
    ES_S32 maxBoxesPerBatch;               /**< Maximum number of boxes to retain for the entire batch after NMS. */
    ES_FLOAT scoreThreshold;               /**< Minimum score threshold to retain a detection. */
    ES_FLOAT iouThreshold;                 /**< IOU threshold for NMS to filter overlapping boxes. */
    ES_FLOAT softNmsSigma;                 /**< Sigma value for soft NMS to adjust scores of overlapping boxes. */
    ES_FLOAT effecImgOffsetX;              /**< Effective offset in the X direction for adjusting box coordinates. */
    ES_FLOAT effecImgOffsetY;              /**< Effective offset in the Y direction for adjusting box coordinates. */
} ES_DETECTION_OUT_CFG;

#define AFFINE_MATRIX_SIZE 9

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif

#endif
